<?php

// A simple class to return a string wrapped in ANSI color codes for terminal output
class Color {
	const RESET = "\033[0m";
	const BOLD = "\033[1m";
	const UNDERLINE = "\033[4m";
	const INVERSE = "\033[7m";
	const BLACK = "\033[30m";
	const RED = "\033[31m";
	const GREEN = "\033[32m";
	const YELLOW = "\033[33m";
	const BLUE = "\033[34m";
	const MAGENTA = "\033[35m";
	const CYAN = "\033[36m";
	const WHITE = "\033[37m";

	private static function format(string $color, string $string): string {
		return $color . $string . self::RESET;
	}

	public static function bold(string $string): string {
		return self::format(self::BOLD, $string);
	}

	public static function underline(string $string): string {
		return self::format(self::UNDERLINE, $string);
	}

	public static function inverse(string $string): string {
		return self::format(self::INVERSE, $string);
	}

	public static function black(string $string): string {
		return self::format(self::BLACK, $string);
	}

	public static function red(string $string): string {
		return self::format(self::RED, $string);
	}

	public static function green(string $string): string {
		return self::format(self::GREEN, $string);
	}

	public static function yellow(string $string): string {
		return self::format(self::YELLOW, $string);
	}

	public static function blue(string $string): string {
		return self::format(self::BLUE, $string);
	}

	public static function magenta(string $string): string {
		return self::format(self::MAGENTA, $string);
	}

	public static function cyan(string $string): string {
		return self::format(self::CYAN, $string);
	}

	public static function white(string $string): string {
		return self::format(self::WHITE, $string);
	}
}

function echoMemoryAndTime(int $i, float $start) {
	echo "(".Color::green("$i lookups").") M: " .
		Color::blue(round(memory_get_usage() / 1024, 1) . " kb") .
		" - T: ". Color::blue(number_format(microtime(true) - $start, 10) ." s") .
		"\n";
}

function echoTitle(string $title) {
	echo "\n";
	echo Color::bold(Color::black("===============================================================")) . "\n";
	echo "\n";
	echo Color::bold(Color::blue($title))."\n";
	echo "\n";
	echo Color::bold(Color::black("===============================================================")) . "\n";
	echo "\n";
}

function readAndAddRoutes(string $file, &$r): array
{
	$array = [];
	$routes = file($file);
	foreach ($routes as $route) {
		[$method, $path] = explode(' ', $route);
		$path = trim($path);
		$array[] = [$method, $path];
		$r->add($method, $path, function() {
			return true;
		});
	}
	return $array;
}

function runIterations(int $iterations, $r, array $routes) {
	echo "Running $iterations iterations\n";
	$start = microtime(true);
	$interval = $iterations / 10;
	for ($i = 0; $i < $iterations; $i++) {
		// pick a random route from the array
		[$method, $uri] = $routes[array_rand($routes)];
		$res = $r->lookup($method, $uri);
		if ($res !== 200) {
			echo Color::red("Failed to handle request for $uri - $res\n");
			exit(1);
		}
		if ($i !== 0 && $i % ($interval) === 0) echoMemoryAndTime($i, $start);
	}
	echo "Time: " . Color::cyan(number_format(microtime(true) - $start, 10) . " s\n");
	// echo the average time per request
	echo "Avg/lookup: " . Color::yellow(number_format((microtime(true) - $start) / $iterations, 10) . " s\n");
	echo "\n";
}
